/*
#############################################################################

 Copyright (c) 2002, 2006 Nexedi SARL and Contributors. All Rights Reserved.
                    Jacek Medrzycki <jacek@erp5.pl>

 WARNING: This program as such is intended to be used by professional
 programmers who take the whole responsability of assessing all potential
 consequences resulting from its eventual inadequacies and bugs
 End users who are looking for a ready-to-use solution with commercial
 garantees and support are strongly adviced to contract a Free Software
 Service Company

 This program is Free Software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#############################################################################
*/

package pl.erp5.ood;

/* Simple java client for oood sever
 *
 * required libraries: xmlrpc-client-3.0.jar, xmlrpc-common-3.0.jar, ws-commons-util-1.0.1.jar
 * http://ws.apache.org/xmlrpc
 */


import org.apache.xmlrpc.client.*;
import org.apache.xmlrpc.*;
import org.apache.ws.commons.util.Base64.*;
import org.apache.ws.commons.util.*;

import java.net.*;
import java.io.*;
import java.util.*;

/**
 * Simple oood client
 */
public class OooDaemonClient {

    private String server_url;
    
    /**
     * Constructs OooDaemonClient instance
     * @param serverUrl URL to the oood server
     */
    public OooDaemonClient(String serverUrl){
        this.server_url = serverUrl;
    }
    
    /**
     * 
     * @param inData data to be converted. Probably read from the disk file.
     * @param extension format to which data should be converted to. E.g. "pdf" would convert to pdf format.
     * @return Returns byte array representing converted data.
     * @throws pl.erp5.ood.OooDaemonException If the conversion could not be carried out. See oood documentation for details.
     * @throws org.apache.xmlrpc.XmlRpcException When XmlRpc problem occurs.
     * @throws java.net.MalformedURLException When the server url is not valid url.
     * @throws org.apache.ws.commons.util.Base64.DecodingException When Base64 conversion fails.
     */
    public byte[] generate (byte[] inData, String extension) 
               throws OooDaemonException, 
                      XmlRpcException,
                      MalformedURLException,
                      DecodingException{
           
        String encData = Base64.encode(inData);
        XmlRpcClientConfigImpl config = new XmlRpcClientConfigImpl();
        config.setServerURL(new URL(server_url));            

        XmlRpcClient client = new XmlRpcClient();
        client.setConfig(config);

        HashMap params = new HashMap();
        params.put("extension",extension);
        params.put("data",encData);

        Vector paramVector = new Vector(1);
        paramVector.add(params);

        Object[] result =
             (Object[])client.execute("generate", paramVector);

        int resultCode = ((Integer)result[0]).intValue();
        if (resultCode == 200){
          encData = (String)((Map)result[1]).get("data");
          return Base64.decode(encData);
        }else{
          throw new OooDaemonException((String)result[2],resultCode);
        }

    }
}
