#!/usr/bin/python
##############################################################################
#
# Copyright (c) 2002, 2006 Nexedi SARL and Contributors. All Rights Reserved.
#                    Bartlomiej Gorny <bartek@erp5.pl>
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

"""
  Test various oood operations - getting and setting metadata, converting basic file types
  to ODF, generating basic file formats
"""

import sys, base64, unittest
import getopt
from xmlrpclib import *
import config
from oood_common import responseFactory, Request

enc = base64.encodestring

try:
  f = open('doc/test.odt')
except IOError:
  print "you need a doc subdir with appropriate documents"
  sys.exit(1)

opts, args = getopt.getopt(sys.argv[1:], '', ["offset=", "port=", "host="])

port = None
host = None

offset_str = ''
offset = 0

for o, a in opts:
  if o == '--offset':
    offset_str = a
    offset = int(a)
  elif o == '--port':
    port = int(a)
  elif o == '--host':
    host = a

doc_dir = 'doc%s' % offset_str

if not port:
  port = config.server_port + offset
  print port
if not host:
  host = config.server_host
  print host

try:
  f = open('%s/test.odt' % doc_dir)
except IOError:
  print "you need a %s subdir with appropriate documents" % doc_dir
  sys.exit(1)

sp = ServerProxy('http://%s:%d' % (host, port))

class TestFormats(unittest.TestCase):

  def testTextFormats(self):
    self.runTestForType('text', 'doc/test.odt')

  def testSpreadsheetFormats(self):
    self.runTestForType('spreadsheet', 'doc/test.ods')

  def testPresentationFormats(self):
    self.runTestForType('presentation', 'doc/test.odp')

  def testDrawingFormats(self):
    self.runTestForType('drawing', 'doc/test.odg')

  def runTestForType(self, type, filename):
    data = open(filename).read()
    failed_format_list = []
    request = Request(document_type = type)
    res = sp.getAllowedTargetItemList(request)
    extension_list = [x[0] for x in res[1]['response_data']]
    print '\ntesting %s, generating %s formats...' % (type.title(), len(extension_list))
    for extension in extension_list:
      request = Request(filename=filename, extension=extension, data=data)
      res = sp.generate(request)
      res = responseFactory(res)
      if res.code == 200:
        msg = 'PASSED'
        newname = 'doc/out/test_changed.%s' % extension
        if res.mime == 'application/zip':
          newname += '.zip'
        open(newname, 'w').write(res.data)
      else:
        msg = 'FAILED (%s)' % res.code
        failed_format_list.append(extension)
      print '%-15s\t...%s' % (extension, msg)
    self.failIf(failed_format_list, 'Failed for %s on: %s' % (type.title(), ', '.join(failed_format_list)))


if __name__=='__main__':
  tests=(TestFormats,)
  for t in tests:
    suite = unittest.makeSuite(t)
    unittest.TextTestRunner(verbosity=2).run(suite)




# vim: filetype=python syntax=python shiftwidth=2 
