#!/usr/bin/python
##############################################################################
#
# Copyright (c) 2002, 2006 Nexedi SARL and Contributors. All Rights Reserved.
#                    Bartlomiej Gorny <bartek@erp5.pl>
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

"""
  Try to convert some older OpenOffice and StarOffice file types
  to current ODF formats
"""

import sys, base64, unittest
import getopt
from xmlrpclib import *
sys.path.append('/etc/oood')
import config
from oood_common import responseFactory

enc = base64.encodestring

opts, args = getopt.getopt(sys.argv[1:], '', ["offset=", "port=", "host="])

port = None
host = None

offset_str = ''
offset = 0

for o, a in opts:
  if o == '--offset':
    offset_str = a
    offset = int(a)
  elif o == '--port':
    port = int(a)
  elif o == '--host':
    host = a

doc_dir = 'doc%s' % offset_str

if not port:
  port = config.server_port + offset
  print port
if not host:
  host = config.server_host
  print host

try:
  f = open('%s/test.odt' % doc_dir, 'rb')
except IOError:
  print "you need a %s subdir with appropriate documents" % doc_dir
  sys.exit(1)

sp = ServerProxy('http://%s:%d' % (host, port))


class TestFileOperations(unittest.TestCase):

  def test_sxc2odf(self):
    self.convertFile('sxc', 'ods', 's2o')

  def test_sdc2odf(self):
    self.convertFile('sdc', 'ods', 'sdc2o')

  def test_sxw2odf(self):
    self.convertFile('sxw', 'odt', 'sxw2o')

  def test_sdw2odf(self):
    self.convertFile('sdw', 'odt', 'sdw2o')

  def test_sxi2odf(self):
    self.convertFile('sxi', 'odp', 'sxi2o')

  def test_sdd2odf(self):
    self.convertFile('sdd', 'odp', 'sdd2o')

  def test_sxd2odf(self):
    self.convertFile('sxd', 'odg', 'sxd2o')

  def test_sda2odf(self):
    self.convertFile('sda', 'odg', 'sda2o')

  def test_odg2svn(self):
    self.generateFile('odg', 'svg')

  def convertFile(self, src, ext, id=''):
    if id: id = id + '.'
    data=open('doc/in/test.%s%s' % (id, src), 'rb').read()
    res=sp.convert('test.%s%s' % (id, src),enc(data))
    res = responseFactory(res)
    self.assert_(res)
    open('doc/out/test.%sout.%s' % (id, ext), 'wb').write(res.data)

  def generateFile(self, src, ext, id=''):
    if id: id = id + '.'
    data=open('doc/in/test.%s%s' % (id, src), 'rb').read()
    res=sp.generate('test.%s%s' % (id, src), enc(data), -1, ext)
    res = responseFactory(res)
    self.assert_(res)
    open('doc/out/test.%sout.%s' % (id, ext), 'wb').write(res.data)

if __name__=='__main__':
  tests=(TestFileOperations,)
  for t in tests:
    suite=unittest.makeSuite(t)
    unittest.TextTestRunner(verbosity=2).run(suite)




# vim: filetype=python syntax=python shiftwidth=2 
