##############################################################################
#
# Copyright (c) 2002, 2007 Nexedi SA and Contributors. All Rights Reserved.
#                    Klaus Woelfel 
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

import os
import sys
import config

sys.path.append(config.uno_path)
os.environ['PATH'] += ';' + config.uno_path + ';'

import socket
import start
import pywintypes
import win32api
import win32con
import win32process
import win32event
import win32service
import win32serviceutil
import win32com.client
from time import sleep
from logger import Log
from dispatcher import Dispatcher, MySerw

class ooodService(win32serviceutil.ServiceFramework):
  _svc_name_ = 'ERP5oood'
  _svc_display_name_ = 'ERP5oood'
  _svc_description_ =  ('A server for converting docs from other formats to '
                        'OOo, generating pdf files, getting and setting '
                        'metadata. To be used together with ERP5.')
  _exe_name_ = os.path.join(os.path.dirname(sys.executable),'..','lib',
      'site-packages','win32','pythonservice.exe')

  def __init__(self, args):
    # set the timeout so the service can stop...Otherwise it hangs forever
    socket.setdefaulttimeout(15)
    win32serviceutil.ServiceFramework.__init__(self, args)
    self.hWaitStop = win32event.CreateEvent(None, 0, 0, None)
    
  def SvcStop(self):
    self.ReportServiceStatus(win32service.SERVICE_STOP_PENDING)
    win32event.SetEvent(self.hWaitStop)
    
  def SvcDoRun(self):
    if not os.access(config.tmp_dir, os.W_OK):
      message = 'ERROR: A oood tmp directory must exist and must be writeable!!!'
      Log.info(message)
      return
    message = "Server - Started as Windows Service"
    Log.info(message)
    # Start the server core
    disp = Dispatcher()
    server_port = config.server_port
    ser = MySerw((config.server_host, server_port))
    ser.register_instance(disp)
    # handle requests until Service is stopped
    while True:
      rc = win32event.WaitForSingleObject(self.hWaitStop, 0)
      if rc == win32event.WAIT_TIMEOUT:
        ser.handle_request()
      else:
        # kill OpenOffice insatnces
        start.flushPool()
        break

def getChildrenOfPid(pid):
  wmi = win32com.client.GetObject('winmgmts:')
  children = wmi.ExecQuery('Select * from win32_process where ParentProcessId=%s' % pid)
  pids = []
  for proc in children:
    pids.append(int(proc.Properties_('ProcessId')))
  return pids

def SafeTerminateProcess(pid, timeout=5000):
  success = False
  # if opening the handle failed, than the process is already killed and
  # we return success also we did nothing
  try:
    handle = win32api.OpenProcess(win32con.PROCESS_ALL_ACCESS, False, pid)
  except pywintypes.error, e:
    if e[0] == 87:
      return success
    else:
      raise
    
  exitcode = win32process.GetExitCodeProcess(handle)
  if exitcode == win32con.STILL_ACTIVE:
    hKernel = win32api.GetModuleHandle("Kernel32")
    procExit = win32api.GetProcAddress(hKernel, "ExitProcess")
    hRemoteT = win32process.CreateRemoteThread(handle, None, 0, procExit, -1, 0)
    # waiting after starting remote thread
    retval = win32event.WaitForSingleObject(handle, timeout)
    # if timeout elapsed, we can't guarantee that the process is dead
    if retval != win32con.WAIT_TIMEOUT:
      success = True
    #win32api.CloseHandle(hRemoteT) #doesn't work at the moment
  else:
    # the process was already dead
    success = True
  win32api.CloseHandle(handle)
  return success
  
if __name__=='__main__':
 win32serviceutil.HandleCommandLine(ooodService)
